@@ -2,6 +2,8 @@ from event_stream.models.model import *
 from sqlalchemy import Table, Column, MetaData, create_engine
 import os
 import urllib
+import psycopg2
+from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import sessionmaker
 
 
@@ -18,7 +20,8 @@ class DAO(object):
         DATABASE_URL = 'postgresql://{}:{}@{}:{}/{}'.format(db_username, db_password, host_server,
                                                             db_server_port, database_name)
         print(DATABASE_URL)
-        engine = create_engine('postgresql+psycopg2://streams:REPLACE_ME@postgres:5432/amba')
+        # engine = create_engine('postgresql+psycopg2://streams:REPLACE_ME@postgres:5432/amba')
+        engine = create_engine(DATABASE_URL)
         Base.metadata.create_all(engine)
         # database = databases.Database(DATABASE_URL)
 
@@ -26,20 +29,33 @@ class DAO(object):
         self.session = Session()
 
     def save_object(self, obj):
-        self.session.add(obj)
-        self.session.commit()
-        self.session.refresh(obj)
+        self.session.merge(obj)
+        try:
+            self.session.commit()
+        except IntegrityError:
+            self.session.rollback()
+        # self.session.refresh(obj)
         return obj
 
     def get_object(self, table, key):
-        return self.session.query(table).filter_by(**key).first()
+        try:
+            result = self.session.query(table).filter_by(**key).first()
+        except psycopg2.errors.UniqueViolation as ex:
+            print(ex)
+        else:
+            return result
+        finally:
+            return None
 
     def save_if_not_exist(self, obj, table, kwargs):
-        obj_db = self.get_object(table, kwargs)
-        if obj_db.count() > 0:
-            return self.save_object(obj)
-        else:
-            return obj_db
+        if not obj.id:
+            obj_db = self.get_object(table, kwargs)
+            if obj_db:
+                print('does exists')
+                return obj_db
+            else:
+                print('does not exists')
+                return self.save_object(obj)
 
     def get_publication(self, doi):
         return self.get_object(Publication, {'doi': doi})
@@ -60,14 +76,14 @@ class DAO(object):
 
             author = self.save_if_not_exist(author, Author, {'normalizedName': author.normalizedName})
 
-            publication_authors = PublicationAuthor({'authorId': author.id, 'publicationId': publication.id})
+            publication_authors = PublicationAuthor(**{'authorId': author.id, 'publicationDoi': publication.doi})
             self.save_object(publication_authors)
 
         sources = publication_data['source_id']
         for sources_data in sources:
             source = Source(title=sources_data['title'], url=sources_data['url'])
             source = self.save_if_not_exist(source, Source, {'title': source.title})
-            publication_sources = PublicationSource({'sourceId': source.id, 'publicationId': publication.id})
+            publication_sources = PublicationSource(**{'sourceId': source.id, 'publicationDoi': publication.doi})
             self.save_object(publication_sources)
 
         fields_of_study = publication_data['fieldOfStudy']
@@ -75,7 +91,7 @@ class DAO(object):
             fos = FieldOfStudy(name=fos_data['name'], normalizedName=fos_data['normalizedName'])
             fos.level = 2
             fos = self.save_if_not_exist(fos, FieldOfStudy, {'normalizedName': fos.normalizedName})
-            publication_fos = PublicationFieldOfStudy({'fieldOfStudyId': fos.id, 'publicationId': publication.id})
+            publication_fos = PublicationFieldOfStudy(**{'fieldOfStudyId': fos.id, 'publicationDoi': publication.doi})
             self.save_object(publication_fos)
 
         return publication
